@@ -4,7 +4,7 @@ package Text::ASCIITable;
 @ISA=qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw();
-$VERSION = '0.19';
+$VERSION = '0.21';
 use Exporter;
 use strict;
 use Carp;
@@ -13,6 +13,8 @@ use overload '@{}' => 'addrow_overload', '""' => 'drawit';
 use Encode;
 use List::Util qw(reduce max sum);
 
+=encoding utf8
+
 =head1 NAME
 
 Text::ASCIITable - Create a nice formatted table using ASCII characters.
@@ -88,7 +90,6 @@ sub new {
   $self->{options}{alignHeadRow} = $self->{options}{alignHeadRow} || 'auto'; # default setting
   $self->{options}{undef_as} = $self->{options}{undef_as} || ''; # default setting
   $self->{options}{chaining} = $self->{options}{chaining} || 0; # default setting
-  $self->{options}{utf8} = defined($self->{options}{utf8}) ? $self->{options}{utf8} : 1; # default setting
 
   bless $self;
 
@@ -116,8 +117,8 @@ sub setCols {
   my @out;
   grep {$max = scalar(@{$_}) if scalar(@{$_}) > $max} @lines;
   foreach my $num (0..($max-1)) {
-    my @tmp = map { @{$_}[$num] || '' } @lines;
-    push @out, [ @tmp ];
+    my @tmp = map defined $$_[$num] && $$_[$num], @lines;
+    push @out, \@tmp;
   }
 
   @{$self->{tbl_cols}} = @_;
@@ -165,7 +166,7 @@ DBI::selectall_arrayref($sql) without changes.
 sub addRow {
   my $self = shift;
   @_ = @{$_[0]} if (ref($_[0]) eq 'ARRAY');
-  do { $self->reperror("Received too many columns"); return $self->{options}{chaining} ? $self : 1; } if scalar(@_) > scalar(@{$self->{tbl_cols}});
+  do { $self->reperror("Received too many columns"); return $self->{options}{chaining} ? $self : 1; } if scalar(@_) > scalar(@{$self->{tbl_cols}}) && ref($_[0]) ne 'ARRAY';
   my (@in,@out,@lines,$max);
 
 	if (scalar(@_) > 0 && ref($_[0]) eq 'ARRAY') {
@@ -452,7 +453,7 @@ to use HTML tags to for example color the text inside the rows, and you want the
 =item allowANSI
 
 If you use ANSI codes like <ESC>[1mHi this is bold<ESC>[m or similar. This option will make the table to be
-displayed correct when showed in a ANSI compilant terminal. Set this to 1 to enable. There is an example of ANSI support
+displayed correct when showed in a ANSI compliant terminal. Set this to 1 to enable. There is an example of ANSI support
 in this package, named ansi-example.pl.
 
 =item alignHeadRow
@@ -471,7 +472,7 @@ of this line in the draw() function.
 =item headingText
 
 Add a heading above the columnnames/rows wich uses the whole width of the table to output
-a heading/title to the table. The heading-part of the table is automaticly shown when
+a heading/title to the table. The heading-part of the table is automatically shown when
 the headingText option contains text. B<Note:> If this text is so long that it makes the
 table wider, it will not hesitate to change width of columns that have "strict width".
 
@@ -884,7 +885,6 @@ sub count {
   $str =~ s/<.+?>//g if $self->{options}{allowHTML};
   $str =~ s/\33\[(\d+(;\d+)?)?[musfwhojBCDHRJK]//g if $self->{options}{allowANSI}; # maybe i should only have allowed ESC[#;#m and not things not related to
   $str =~ s/\33\([0B]//g if $self->{options}{allowANSI};                           # color/bold/underline.. But I want to give people as much room as they need.
-  $str = decode("utf8", $str) if $self->{options}{utf8};
 
   return length($str);
 }
@@ -908,13 +908,17 @@ sub align {
   } elsif ($dir =~ /right/i) {
     my $visuallen = $self->count($text);
     my $reallen = length($text);
-    $text = (" " x ($length - $visuallen)).$text;
+    if ($length - $visuallen > 0) {
+      $text = (" " x ($length - $visuallen)).$text;
+    }
     return substr($text,0,$length - ($visuallen-$reallen)) if ($strict);
     return $text;
   } elsif ($dir =~ /left/i) {
     my $visuallen = $self->count($text);
     my $reallen = length($text);
-    $text = $text.(" " x ($length - $visuallen));
+    if ($length - $visuallen > 0) {
+      $text = $text.(" " x ($length - $visuallen));
+    }
     return substr($text,0,$length - ($visuallen-$reallen)) if ($strict);
     return $text;
   } elsif ($dir =~ /justify/i) {
@@ -951,7 +955,7 @@ sub align {
     # Someone tell me if this is matematecally totally wrong. :P
     $left = int($left) + 1 if ($left != int($left) && $left > 0.4);
     my $right = int(( $length - $visuallen ) / 2);
-    $text = (" " x $left).$text.(" " x $right);
+    $text = ($left > 0 ? " " x $left : '').$text.($right > 0 ? " " x $right : '');
     return substr($text,0,$length) if ($strict);
     return $text;
   } else {
@@ -992,6 +996,7 @@ sub PUSH {
 
 sub reperror {
   my $self = shift;
+  print STDERR Carp::shortmess(shift) if $self->{options}{reportErrors};
 }
 
 # Best way I could think of, to search the array.. Please tell me if you got a better way.
@@ -1071,11 +1076,11 @@ Hkon Nessjen, <lunatic@cpan.org>
 
 =head1 VERSION
 
-Current version is 0.19.
+Current version is 0.20.
 
 =head1 COPYRIGHT
 
-Copyright 2002-2003 by Hkon Nessjen.
+Copyright 2002-2011 by Hkon Nessjen.
 All rights reserved.
 This module is free software;
 you can redistribute it and/or modify it under the same terms as Perl itself.
